# -*- coding: utf-8 -*-
"""This module contains the controller classes of the application."""

# symbols which are imported by "from pagedemo1.controllers import *"
__all__ = ['Root']

# standard library imports
# import logging
import datetime

# third-party imports
from genshi import Markup

from turbogears import controllers, expose, flash, paginate
from turbogears.widgets import PaginateDataGrid

# project specific imports
from pagedemo1.model import Person

# The datagrids used for this demo:

person_list = PaginateDataGrid(
    fields=[
        PaginateDataGrid.Column('name', 'name', 'Name'),
        PaginateDataGrid.Column('age', 'age', 'Age'),
    ])

sortable_person_list = PaginateDataGrid(
    fields=[
        PaginateDataGrid.Column('name', 'name', 'Name',
            options=dict(sortable=True)),
        PaginateDataGrid.Column('age', 'age', 'Age',
            options=dict(sortable=True, reverse_order=True)),
    ])

class MakeLink(object):
    """Generate the link inside the datagrid."""

    def __init__(self, baseurl, id, title, action):
        self.baseurl = baseurl
        self.id = id
        self.title = title
        self.action = action

    def __call__(self, obj):
        url = controllers.url(self.baseurl,
            dict(id=obj[self.id], action=self.action))
        text = obj[self.title]
        return Markup('<a href="%s" style="text-decoration: underline">%s</a>'
            % (url, text))

# to get this kind of link:  http://localhost:8080/person?action=edit&id=6
mylink = MakeLink('person', 'id', 'name', 'edit')

# just to get a element from a dictionary
def get(fieldname):
    return lambda x: x.get(fieldname)

data_grid = PaginateDataGrid(
    fields = [
        PaginateDataGrid.Column(name='name', getter=mylink, title='Name',
            options=dict(sortable=True)),
        PaginateDataGrid.Column(name='age', getter=get('age'), title='Age',
            options=dict(sortable=True, reverse_order=True)),
    ])


# log = logging.getLogger("pagedemo1.controllers")


class Root(controllers.RootController):
    """The root controller of the application."""

    @expose(template='pagedemo1.templates.welcome')
    def index(self):
        flash(_(u"The Paginate Demo 1 is now running"))
        return dict(now=datetime.datetime.now())

    # paginate1 will paginate using tg.paginate
    @expose(template='pagedemo1.templates.paginate1')
    @paginate('persons')
    def paginate1(self):
        return dict(persons=Person.query)

    # paginate2 will paginate automatically using PaginateDataGrid
    @expose(template='pagedemo1.templates.paginate2')
    @paginate('persons')
    def paginate2(self):
        return dict(persons=Person.query, list=person_list)

    # paginate3 is like paginate2 except it has sorting support
    @expose(template='pagedemo1.templates.paginate2')
    @paginate('persons', default_order='name')
    def paginate3(self):
        return dict(persons=Person.query, list=sortable_person_list)

    # paginate4 is like paginate3 except it does not use SQL
    @expose(template='pagedemo1.templates.paginate2')
    @paginate('persons', default_order='name')
    def paginate4(self):
        return dict(persons=self.persons(), list=data_grid)

    # paginate5 is like paginate4 except it does not use PaginateDataGrid
    @expose(template='pagedemo1.templates.paginate5')
    @paginate('persons')
    def paginate5(self):
        return dict(persons=self.persons())

    def persons(self):
        return [dict(id=i, name='name%d'%i, age=100-i) for i in range(100)]

    @expose(template='pagedemo1.templates.person')
    def person(self, id, action):
        person = self.persons()[int(id)]
        return dict(person=person)

    # add some persons
    @expose(template='pagedemo1.templates.welcome')
    def populate(self):
        if Person.query.first():
            flash(_(u"The Person table has already been populated"))
        else:
            for i in range(1, 100):
                person = Person()
                person.name = 'dummy%i' % i
                person.age = i
            flash(_(u"The Person table has been populated with 100 entries"))
        return dict(now=datetime.datetime.now())
